@@ -10,28 +10,33 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import uuid
+
 import httpretty
 
 from keystoneclient.tests.v2_0 import utils
+from keystoneclient.v2_0 import roles
 from keystoneclient.v2_0 import users
 
 
 class UserTests(utils.TestCase):
     def setUp(self):
         super(UserTests, self).setUp()
+        self.ADMIN_USER_ID = uuid.uuid4().hex
+        self.DEMO_USER_ID = uuid.uuid4().hex
         self.TEST_USERS = {
             "users": {
                 "values": [
                     {
                         "email": "None",
                         "enabled": True,
-                        "id": 1,
+                        "id": self.ADMIN_USER_ID,
                         "name": "admin",
                     },
                     {
                         "email": "None",
                         "enabled": True,
-                        "id": 2,
+                        "id": self.DEMO_USER_ID,
                         "name": "demo",
                     },
                 ]
@@ -40,11 +45,13 @@ class UserTests(utils.TestCase):
 
     @httpretty.activate
     def test_create(self):
+        tenant_id = uuid.uuid4().hex
+        user_id = uuid.uuid4().hex
         req_body = {
             "user": {
                 "name": "gabriel",
                 "password": "test",
-                "tenantId": 2,
+                "tenantId": tenant_id,
                 "email": "test@example.com",
                 "enabled": True,
             }
@@ -54,8 +61,8 @@ class UserTests(utils.TestCase):
             "user": {
                 "name": "gabriel",
                 "enabled": True,
-                "tenantId": 2,
-                "id": 3,
+                "tenantId": tenant_id,
+                "id": user_id,
                 "password": "test",
                 "email": "test@example.com",
             }
@@ -69,29 +76,31 @@ class UserTests(utils.TestCase):
                                         tenant_id=req_body['user']['tenantId'],
                                         enabled=req_body['user']['enabled'])
         self.assertIsInstance(user, users.User)
-        self.assertEqual(user.id, 3)
+        self.assertEqual(user.id, user_id)
         self.assertEqual(user.name, "gabriel")
         self.assertEqual(user.email, "test@example.com")
         self.assertRequestBodyIs(json=req_body)
 
     @httpretty.activate
     def test_create_user_without_email(self):
+        tenant_id = uuid.uuid4().hex
         req_body = {
             "user": {
                 "name": "gabriel",
                 "password": "test",
-                "tenantId": 2,
+                "tenantId": tenant_id,
                 "enabled": True,
                 "email": None,
             }
         }
 
+        user_id = uuid.uuid4().hex
         resp_body = {
             "user": {
                 "name": "gabriel",
                 "enabled": True,
-                "tenantId": 2,
-                "id": 3,
+                "tenantId": tenant_id,
+                "id": user_id,
                 "password": "test",
             }
         }
@@ -104,23 +113,57 @@ class UserTests(utils.TestCase):
             tenant_id=req_body['user']['tenantId'],
             enabled=req_body['user']['enabled'])
         self.assertIsInstance(user, users.User)
-        self.assertEqual(user.id, 3)
+        self.assertEqual(user.id, user_id)
         self.assertEqual(user.name, "gabriel")
         self.assertRequestBodyIs(json=req_body)
 
+    @httpretty.activate
+    def test_create_user_without_password(self):
+        user_name = 'test'
+        user_id = uuid.uuid4().hex
+        tenant_id = uuid.uuid4().hex
+        user_enabled = True
+        req_body = {
+            'user': {
+                'name': user_name,
+                'password': None,
+                'tenantId': tenant_id,
+                'enabled': user_enabled,
+                'email': None,
+            }
+        }
+        resp_body = {
+            'user': {
+                'name': user_name,
+                'enabled': user_enabled,
+                'tenantId': tenant_id,
+                'id': user_id,
+            }
+        }
+
+        self.stub_url(httpretty.POST, ['users'], json=resp_body)
+
+        user = self.client.users.create(user_name, tenant_id=tenant_id,
+                                        enabled=user_enabled)
+        self.assertIsInstance(user, users.User)
+        self.assertEqual(user_id, user.id)
+        self.assertEqual(user_name, user.name)
+        self.assertRequestBodyIs(json=req_body)
+
     @httpretty.activate
     def test_delete(self):
-        self.stub_url(httpretty.DELETE, ['users', '1'], status=204)
-        self.client.users.delete(1)
+        self.stub_url(httpretty.DELETE, ['users', self.ADMIN_USER_ID],
+                      status=204)
+        self.client.users.delete(self.ADMIN_USER_ID)
 
     @httpretty.activate
     def test_get(self):
-        self.stub_url(httpretty.GET, ['users', '1'],
+        self.stub_url(httpretty.GET, ['users', self.ADMIN_USER_ID],
                       json={'user': self.TEST_USERS['users']['values'][0]})
 
-        u = self.client.users.get(1)
+        u = self.client.users.get(self.ADMIN_USER_ID)
         self.assertIsInstance(u, users.User)
-        self.assertEqual(u.id, 1)
+        self.assertEqual(u.id, self.ADMIN_USER_ID)
         self.assertEqual(u.name, 'admin')
 
     @httpretty.activate
@@ -162,47 +205,51 @@ class UserTests(utils.TestCase):
     def test_update(self):
         req_1 = {
             "user": {
-                "id": 2,
+                "id": self.DEMO_USER_ID,
                 "email": "gabriel@example.com",
                 "name": "gabriel",
             }
         }
         req_2 = {
             "user": {
-                "id": 2,
+                "id": self.DEMO_USER_ID,
                 "password": "swordfish",
             }
         }
+        tenant_id = uuid.uuid4().hex
         req_3 = {
             "user": {
-                "id": 2,
-                "tenantId": 1,
+                "id": self.DEMO_USER_ID,
+                "tenantId": tenant_id,
             }
         }
         req_4 = {
             "user": {
-                "id": 2,
+                "id": self.DEMO_USER_ID,
                 "enabled": False,
             }
         }
 
-        self.stub_url(httpretty.PUT, ['users', '2'], json=req_1)
-        self.stub_url(httpretty.PUT, ['users', '2', 'OS-KSADM', 'password'],
+        self.stub_url(httpretty.PUT, ['users', self.DEMO_USER_ID], json=req_1)
+        self.stub_url(httpretty.PUT,
+                      ['users', self.DEMO_USER_ID, 'OS-KSADM', 'password'],
                       json=req_2)
-        self.stub_url(httpretty.PUT, ['users', '2', 'OS-KSADM', 'tenant'],
+        self.stub_url(httpretty.PUT,
+                      ['users', self.DEMO_USER_ID, 'OS-KSADM', 'tenant'],
                       json=req_3)
-        self.stub_url(httpretty.PUT, ['users', '2', 'OS-KSADM', 'enabled'],
+        self.stub_url(httpretty.PUT,
+                      ['users', self.DEMO_USER_ID, 'OS-KSADM', 'enabled'],
                       json=req_4)
 
-        self.client.users.update(2,
+        self.client.users.update(self.DEMO_USER_ID,
                                  name='gabriel',
                                  email='gabriel@example.com')
         self.assertRequestBodyIs(json=req_1)
-        self.client.users.update_password(2, 'swordfish')
+        self.client.users.update_password(self.DEMO_USER_ID, 'swordfish')
         self.assertRequestBodyIs(json=req_2)
-        self.client.users.update_tenant(2, 1)
+        self.client.users.update_tenant(self.DEMO_USER_ID, tenant_id)
         self.assertRequestBodyIs(json=req_3)
-        self.client.users.update_enabled(2, False)
+        self.client.users.update_enabled(self.DEMO_USER_ID, False)
         self.assertRequestBodyIs(json=req_4)
 
     @httpretty.activate
@@ -215,9 +262,56 @@ class UserTests(utils.TestCase):
         resp_body = {
             'access': {}
         }
-        self.stub_url(httpretty.PATCH, ['OS-KSCRUD', 'users', '123'],
+        user_id = uuid.uuid4().hex
+        self.stub_url(httpretty.PATCH, ['OS-KSCRUD', 'users', user_id],
                       json=resp_body)
 
-        self.client.user_id = '123'
+        self.client.user_id = user_id
         self.client.users.update_own_password('DCBA', 'ABCD')
         self.assertRequestBodyIs(json=req_body)
+
+    @httpretty.activate
+    def test_user_role_listing(self):
+        user_id = uuid.uuid4().hex
+        role_id1 = uuid.uuid4().hex
+        role_id2 = uuid.uuid4().hex
+        tenant_id = uuid.uuid4().hex
+
+        user_resp = {
+            'user': {
+                'id': user_id,
+                'email': uuid.uuid4().hex,
+                'name': uuid.uuid4().hex,
+            }
+        }
+
+        roles_resp = {
+            'roles': {
+                'values': [
+                    {
+                        'name': uuid.uuid4().hex,
+                        'id': role_id1,
+                    },
+                    {
+                        'name': uuid.uuid4().hex,
+                        'id': role_id2,
+                    }
+                ]
+            }
+        }
+
+        self.stub_url(httpretty.GET,
+                      ['users', user_id],
+                      json=user_resp)
+        self.stub_url(httpretty.GET,
+                      ['tenants', tenant_id, 'users', user_id, 'roles'],
+                      json=roles_resp)
+
+        user = self.client.users.get(user_id)
+        role_objs = user.list_roles(tenant_id)
+
+        for r in role_objs:
+            self.assertIsInstance(r, roles.Role)
+
+        self.assertEqual(set([role_id1, role_id2]),
+                         set([r.id for r in role_objs]))
