@@ -20,7 +20,7 @@ from keystoneclient import base
 
 
 class User(base.Resource):
-    """Represents a Keystone user"""
+    """Represents a Keystone user."""
     def __repr__(self):
         return "<User %s>" % self._info
 
@@ -35,6 +35,10 @@ class UserManager(base.ManagerWithFind):
     """Manager class for manipulating Keystone users."""
     resource_class = User
 
+    def __init__(self, client, role_manager):
+        super(UserManager, self).__init__(client)
+        self.role_manager = role_manager
+
     def get(self, user):
         return self._get("/users/%s" % base.getid(user), "user")
 
@@ -86,7 +90,8 @@ class UserManager(base.ManagerWithFind):
         return self._update("/users/%s/OS-KSADM/tenant" % base.getid(user),
                             params, "user")
 
-    def create(self, name, password, email=None, tenant_id=None, enabled=True):
+    def create(self, name, password=None, email=None,
+               tenant_id=None, enabled=True):
         """Create a user."""
         params = {"user": {"name": name,
                            "password": password,
@@ -122,5 +127,5 @@ class UserManager(base.ManagerWithFind):
                               "users")
 
     def list_roles(self, user, tenant=None):
-        return self.api.roles.roles_for_user(base.getid(user),
-                                             base.getid(tenant))
+        return self.role_manager.roles_for_user(base.getid(user),
+                                                base.getid(tenant))
